#!/usr/bin/env python2.7

import os, sys, re

args = sys.argv

if len(args) != 3:
	print "Usage: create_test.py <rawdata file> <report file>"
	print "The .test file is written to stdout."
	sys.exit(1)

def readRawFile():
	rawdata = []
	for line in open(args[1], "rt").readlines():
		lookfor = ["basic_cpuid", "ext_cpuid", "intel_fn4", "intel_fn11"]
		good = False
		for match in lookfor:
			if line.find(match) != -1:
				good = True
				break
		if good:
			rawdata.append(line.strip())
	return rawdata

def readResultFile():
	repdata = []
	rexp = re.compile('(-?[0-9]+).*')
	for line in open(args[2], "rt").readlines():
		s = line.strip()
		if s.find(":") == -1:
			continue
		numeric = ["family", "model", "stepping", "ext_family", "ext_model",
				"num_cores", "num_logical", "L1 D cache", "L1 I cache",
				"L2 cache", "L3 cache", "L4 cache", "L1D assoc.",
				"L2 assoc.", "L3 assoc.", "L4 assoc.", "L1D line sz",
				"L2 line sz", "L3 line sz", "L4 line sz"]
		field = s[:s.find(":")].strip()
		if field in numeric:
			value = s[s.find(":")+1:].strip()
			if not rexp.match(value):
				raise "Bad format of value: [%s]" % s
			repdata.append(rexp.findall(value)[0])
		if field == "code name":
			value = s[s.find("`") + 1: s.find("'")]
			repdata.append(value)
		if field == "features":
			value = s[s.find(":") + 2:]
			repdata.append(value)
		if field == "SSE units":
			value = s[s.find(":") + 2:]
			# the value here is something like "XX bits (authoritative)". We remove the "bits" part:
			i = value.find("bits")
			if i != -1:
				value = value[:i] + value[i + 5:]
			repdata.append(value)
	return repdata

delimiter = "-" * 80
lines = readRawFile() + [delimiter] + readResultFile()
sys.stdout.writelines(map(lambda s: s + "\n", lines))



#!/usr/bin/env python2.7

import os, sys, re, glob

if len(sys.argv) != 2:
	print "Usage: check-consistency <path>"
	sys.exit(0)


def getEnumElements(enumName):
	f = open("%s/libcpuid.h" % sys.argv[1], "r")
	l = []
	on = False
	rexp = re.compile(r'^\s*([A-Z0-9_]+)(\s*=\s*[A-Z0-9_]+)?\s*,.*$')
	for line in f:
		line = line.strip()
		if line.startswith("typedef enum {"):
			l = []
			on = True
		if on and rexp.match(line):
			l.append(rexp.findall(line)[0][0])
		if on and line.startswith("} "):
			on = False
			if line.find(enumName) != -1:
				return l
	f.close()
	return []

def getConstant(constantName):
	f = open("%s/libcpuid_constants.h" % sys.argv[1], "r")
	value = 0
	for line in f:
		items = line.strip().split()
		if len(items) >= 3 and items[0] == "#define" and items[1] == constantName:
			value = int(items[2])
	f.close()
	return value

def checkEnumSize(enumName, constantName):
	print "Checking enum `%s':" % enumName,
	count = len(getEnumElements(enumName)) - 1
	themax = getConstant(constantName)
	print "%d elements; max size (%s=%d)..." % (count, constantName, themax),
	if count > themax:
		print "FAILED"
		global firstError
		firstError = False
	else:
		print "OK"

checkEnumSize("cpu_feature_t", "CPU_FLAGS_MAX")
checkEnumSize("cpu_hint_t", "CPU_HINTS_MAX")
checkEnumSize("cpu_sgx_feature_t", "SGX_FLAGS_MAX")

rexp = re.compile('.*{ CPU_FEATURE_([^,]+), "([^"]+)".*}.*')
print "Finding features:"
for fn in glob.glob("%s/*.c" % sys.argv[1]):
	f = open(fn, "rt")
	line = 1
	nfeat = 0
	for s in f.readlines():
		if rexp.match(s):
			nfeat += 1
			res = rexp.findall(s)
			if len(res) > 1:
				raise "..Too many matches"
			if res[0][0].lower() != res[0][1]:
				print "..Mismatch - %s:%d - `%s' vs `%s'" % (os.path.basename(fn), line, res[0][0], res[0][1])
		line += 1
	if nfeat:
		print "  %s: %d features described" % (os.path.basename(fn), nfeat)
	f.close()

# Check whether all features are converted by cpu_feature_str():

allf = []
f = open("%s/libcpuid.h" % sys.argv[1], "rt")
rexp = re.compile('\t(CPU_FEATURE_[^, ]+).*')
for s in f.readlines():
	if rexp.match(s):
		allf.append(rexp.findall(s)[0])

f.close()

impf = []
rexp = re.compile('\t+{ (CPU_FEATURE_[^,]+).*')
f = open("%s/cpuid_main.c" % sys.argv[1], "rt")
for s in f.readlines():
	if rexp.match(s):
		entry = rexp.findall(s)[0]
		if entry in impf:
			print "cpu_feature_str(): duplicate entry: %s" % entry
		impf.append(entry)
f.close()

print "Found %d total features and %d named features" % (len(allf), len(impf))

for feature in allf:
	if not feature in impf:
		print "cpu_feature_str(): don't have entry for %s" % feature

# Check whether all features have detection code:
print "Checking whether all features have detection code...",
firstError = True

files_code = {}

rexp = re.compile('\t+{ *[0-9]+, (CPU_FEATURE_[^ }]+).*')

for fn in glob.glob("%s/*.c" % sys.argv[1]):
	f = open(fn, "rt")
	files_code[fn] = []
	for s in f.readlines():
		if rexp.match(s):
			entry = rexp.findall(s)[0]
			files_code[fn].append(entry)
	f.close()

for feature in allf:
	matching_files = []
	for fn in files_code:
		if feature in files_code[fn]:
			matching_files.append(fn)
	if len(matching_files) == 0:
		if firstError:
			print "FAILED:"
			firstError = False
		print "..No detection code for %s" % feature
	if len(matching_files) > 1:
		if firstError:
			print "FAILED:"
			firstError = False
		print "..Conflicting detection code for %s in files %s" % (feature, " and ".join(matching_files))

if firstError:
	print "All OK."
print ""

print "Checking processor definitions:"
cache_exp = re.compile(".*([\(/ ][0-9]+K).*")
# Check whether CPU codenames for consistency:
#   - Codenames should not exceed 31 characters
#   - Check for common typos
common_cache_sizes = ["8", "16", "32", "64", "128", "256", "512", "1024", "2048", "3072", "4096", "6144", "8192", "12288", "16384"]
for fn in glob.glob("%s/*.c" % sys.argv[1]):
	bfn = os.path.basename(fn)
	nline = 0
	f = open(fn, "rt")
	has_matchtable = False
	cdefs = 0
	allok = True
	for line in f.readlines():
		nline += 1
		if line.find("struct match_entry_t") != -1:
			has_matchtable = True
		if not has_matchtable:
			continue
		i = line.find("{")
		j = line.find("}")
		if i == -1 or j == -1 or i > j:
			continue
		inner = line[i+1:j]
		parts = inner.split(",")
		if len(parts) == 11: #this number needs to change if the definition of match_entry_t ever changes
			cdefs += 1
			s = parts[10].strip()
			if s[0] != '"' or s[-1] != '"':
				print "..Warning, %s:%d - cannot correctly handle the cpu codename" % (bfn, nline)
				allok = False
				continue
			s = s[1:-1]
			if len(s) > 31:
				print "..%s:%d - codename (%s) is longer than 31 characters!" % (bfn, nline, s)
				allok = False
			if cache_exp.match(s):
				cache_size = cache_exp.findall(s)[0][1:-1]
				if not cache_size in common_cache_sizes:
					print "..Warning, %s:%d - suspicious cache size in codename [%s] (%s)" % (bfn, nline, s, cache_size)
					allok = False
	if cdefs:
		print "  %s: %d processor definitions," % (bfn, cdefs),
		if allok:
			print "all OK"
		else:
			print "some errors/warnings"
	f.close()



#!/usr/bin/env python2.7

from __future__ import with_statement
import os, sys, re, random


### Constants:
fields = [ "family", "model", "stepping", "extfamily", "extmodel", "cores",
	   "logical", "l1d-cache", "l1i-cache", "l2-cache", "l3-cache", "l4-cache",
	   "l1d-assoc", "l2-assoc", "l3-assoc", "l4-assoc", "l1d-cacheline",
	   "l2-cacheline", "l3-cacheline", "l4-cacheline", "sse-size",
	   "codename", "flags" ]

args = sys.argv
fix = False
show_test_fast_warning = False

if len(args) < 3:
	print """
Usage: run_tests.py <cpuid_tool binary> <test file/dir> [test file/dir ...] [OPTIONS]

If a test file is given, it is tested by itself.
If a directory is given, process all *.test files there, subdirectories included.

If the --fix option is given, the behaviour of the cpuid_tool binary is deemed correct
and any failing tests are updated.
"""
	sys.exit(1)

filelist = []
cpuid_tool = args[1]
for arg in args[2:]:
	if arg == "--fix":
		fix = True
		continue
	if arg == "--show-test-fast-warning":
		show_test_fast_warning = True
		continue
	if os.path.isdir(arg):
		# gather all *.test files from subdirs amd and intel:
		for dirpath, dirnames, filenames in os.walk(arg):
			filelist += [os.path.join(dirpath, fn) for fn in filenames if fn[-5:] == ".test"]
	else:
		filelist.append(arg)

#f = open(args[1], "rt")
#lines = f.readlines()
#f.close()

# One would usually use os.tempnam, but libc gives off hell a lot of
# warnings when you attempt to use that :(
def make_tempname(prefix):
	chars = ""
	for i in xrange(26):
		chars += chr(97+i)
		chars += chr(65+i)
	for i in xrange(10):
		chars += chr(48+i)
	for i in xrange(6):
		prefix += random.choice(chars)
	return prefix

def fmt_error(err):
	pfix = "  %s: " % err[0]
	return "%sexpected `%s'\n%sgot      `%s'" % (pfix, err[1], " "*len(pfix), err[2])

def fixFile(filename, input_lines, output_lines):
	f = open(filename, "wt")
	f.writelines(map(lambda s: s + "\n", input_lines))
	f.write("--------------------------------------------------------------------------------\n")
	f.writelines(map(lambda s: s + "\n", output_lines))
	f.close()

def do_test(inp, expected_out, binary, test_file_name):
	fninp = make_tempname("cpuidin")
	fnoutp = make_tempname("cpuidout")
	f = open(fninp, "wt")
	f.writelines(map(lambda s: s + "\n", inp))
	f.close()
	cmd = "%s --load=%s --outfile=%s %s" % (binary, fninp, fnoutp, " ".join(map(lambda s: "--" + s, fields)))
	os.system(cmd)
	os.unlink(fninp)
	real_out = []
	try:
		f = open(fnoutp, "rt")
		real_out = map(lambda s: s.strip(), f.readlines())
		f.close()
		os.unlink(fnoutp)
	except IOError:
		return "Exception"
	if len(real_out) != len(expected_out) or len(real_out) != len(fields):
		return "Unexpected number of records returned"
	err_fields = []
	for i in range(len(real_out)):
		if real_out[i] != expected_out[i]:
			err_fields.append((fields[i], expected_out[i], real_out[i]))
	if not err_fields:
		return "OK"
	else:
		if fix:
			fixFile(test_file_name, inp, real_out)
			return "Mismatch, fixed."
		else:
			return "Mismatch in fields:\n%s" % "\n".join([fmt_error(err) for err in err_fields]) 

errors = False
print "Testing..."
for test_file_name in filelist:
	current_input = []
	current_output = []
	build_output = False
	with open(test_file_name, "rt") as f:
		for line in f.readlines():
			if line[:3] == "---":
				build_output = True
			else:
				if build_output:
					current_output.append(line.strip())
				else:
					current_input.append(line.strip())
		#codename = current_output[len(current_output) - 2]
		result = do_test(current_input, current_output, cpuid_tool, test_file_name)
		print "Test [%s]: %s" % (test_file_name[:-5], result)
		if result != "OK":
			errors = True
		build_output = False

if errors:
	if show_test_fast_warning:
		print """
You're running tests in fast mode; before taking any action on the errors
above, please confirm that the slow mode ('make test-old') also fails."""
else:
	print "All successfull!"
